-- DEFLBOX.MS : Box Deflector

-- Category    : Space Warps
-- Environment : MAX 2.0
-- Description : deflector manipulation
-- Reference   : DEFLBOX.MAX

-- Copyright 1998, EffectWare, Moon-Sun Choi (aeroplus)
-- http://www.chollian.net/~efxware
-- efxware@chollian.net

-- general functions ----------------------------------------------------------

-- node : geometry node (parent)
-- pos : position
-- b : bounce ratio
-- w : width
-- l : length
-- x : x rotation
-- y : y rotation
-- z : z rotation

fn draw_deflector node pos b w l x y z =
(
    local def

    -- create deflector node
    def = deflector bounce:b width:w length:l
    def.position = pos
    def.rotation.x_rotation = x
    def.rotation.y_rotation = y
    def.rotation.z_rotation = z

    -- node & deflector link
    if node != undefined then def.parent = node

    def
)

-- node : geometry node
-- bounce : bounce ratio

fn draw_box_deflector node bounce =
(
    local def_list = #()
    local def
    local pos = [0.0, 0.0, 0.0]
    local ext

    if node == undefined then return #()
	
    ext = node.max - node.min

    -- bottom
    pos = [node.center.x, node.center.y, node.min.z]
    def = draw_deflector node pos bounce ext.x ext.y 0 0 0
    append def_list def
    -- top
    pos = [node.center.x, node.center.y, node.max.z]
    def = draw_deflector node pos bounce ext.x ext.y 0 0 0
    append def_list def

    -- left
    pos = [node.min.x, node.center.y, node.center.z]
    def = draw_deflector node pos bounce ext.z ext.y 0 90 0
    append def_list def
    -- right
    pos = [node.max.x, node.center.y, node.center.z]
    def = draw_deflector node pos bounce ext.z ext.y 0 90 0
    append def_list def

    -- front
    pos = [node.center.x, node.min.y, node.center.z]
    def = draw_deflector node pos bounce ext.x ext.z 90 0 0
    append def_list def
    -- back
    pos = [node.center.x, node.max.y, node.center.z]
    def = draw_deflector node pos bounce ext.x ext.z 90 0 0
    append def_list def

    def_list
)

-- bind spacewarps to particle object
-- part : particle node
-- warps_list : space warps node list

fn bind_spacewarps part warps_list =
(
    local counts

    if part == undefined then return false
	
    counts = warps_list.count
    for i = 1 to counts do
    (
        bindSpaceWarp part warps_list[i]
    )
)

-- hide deflectors
-- node_list : node list
-- hidden : true or false

fn hide_objects node_list hidden =
(
    if hidden == true then
	hide node_list
    else
	unhide node_list
)

-- def_list : deflector node list
-- bounce : bounce ratio

fn change_bounce def_list bounce = 
(
    local counts

    counts = def_list.count
    for i = 1 to counts do
    (
        def_list[i].bounce = bounce
    )
)

-- main program ---------------------------------------------------------------

utility box_deflector "Box Deflector"
(
    local part, node
    local def_list = #()

    group "About"
    (
        label l1 "Box Deflector"
        label l2 "Version 1.0"
        label l3 "Copyright 1998, EffectWare"
        label l4 "Choi, Moon-Sun (aeroplus)"
        label l5 "www.chollian.net/~efxware"
    )

    group "Parameters"
    (
        spinner bounce "Bounce:" range:[0.0, 1000000.0, 1.0] align:#center
        checkbox hid "hide Deflector" checked:false align:#center
        pickbutton pick_node "  Pick Node  "
        label node_name
        pickbutton link_part "  Link Particle  "
        label part_name
    )
	
    on box_deflector open do
    (
        def_list = #()
        node_name.text = "None"
        part_name.text = "None"
    )

    on bounce changed val do
    (
        undo on
        (
            change_bounce def_list bounce.value
        )
    )

    on hid changed state do
    (
        undo on
        (
            hide_objects def_list hid.checked
        )
    )

    on pick_node picked obj do
    (
        node = obj
        node_name.text = node.name

        undo on
        (
            -- create new box deflector
            def_list = draw_box_deflector node bounce.value
			
            bind_space_warps part def_list
            hide_objects def_list hid.checked
        )
    )

    on link_part picked obj do
    (
        part = obj
        part_name.text = part.name
		
        undo on
        (
            bind_space_warps part def_list
            hide_objects def_list hid.checked
        )
    )
)
